/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/lines.h>

#include <InterViews/brush.h>
#include <InterViews/color.h>
#include <InterViews/transformer.h>

//
//
//

Line::Line(const Brush *brush,const Color *stroke,const Color *fill,Coord x1,Coord y1,Coord x2,Coord y2,Transformer *t):
	Graphic(brush,stroke,fill,nil,false,false,2,t)
{
	add_point(x1,y1);
	add_point(x2,y2);
}

Line::~Line()
{
}

Glyph *Line::clone() const
{
	return new Line(_brush,_stroke,_fill,_x[0],_y[0],_x[1],_y[1],_t);
}

ClassId Line::get_classid() const
{
	return LINE_GRAPHIC;
}

boolean Line::is_a(ClassId id) const
{
	return id==LINE_GRAPHIC || Graphic::is_a(id);
}

//
//
//

Polyline::Polyline(const Brush *brush,const Color *stroke,const Color *fill,Coord *x,Coord *y,int n,Transformer *t):
	Graphic(brush,stroke,fill,nil,false,false,n,t)
{
	for(int i=0;i<n;++i)
		add_point(x[i],y[i]);
}

Polyline::~Polyline()
{
}

Glyph *Polyline::clone() const
{
	return new Polyline(_brush,_stroke,_fill,_x,_y,_ctrlpts,_t);
}

ClassId Polyline::get_classid() const
{
	return POLYLINE_GRAPHIC;
}

boolean Polyline::is_a(ClassId id) const
{
	return id==POLYLINE_GRAPHIC || Graphic::is_a(id);
}

//
//
//

Polygon::Polygon(const Brush *brush,const Color *stroke,const Color *fill,Coord *x,Coord *y,int n,Transformer *t):
	Graphic(brush,stroke,fill,nil,true,false,n,t)
{
	int i;

	for(i=0;i<n;++i)
		add_point(x[i],y[i]);
}

Polygon::~Polygon()
{ }

Glyph *Polygon::clone() const
{
	return new Polygon(_brush,_stroke,_fill,_x,_y,_ctrlpts,_t);
}

ClassId Polygon::get_classid() const
{
	return POLYGON_GRAPHIC;
}

boolean Polygon::is_a(ClassId id) const
{
	return id==POLYGON_GRAPHIC || Graphic::is_a(id);
}
